@@ -141,6 +141,6 @@ class WSGIServer(logging_dispatcher, object):
         """
         cutoff = now - self.adj.channel_timeout
         for channel in self.active_channels.values():
-            if (channel.task is None) and channel.last_activity < cutoff:
+            if (not channel.requests) and channel.last_activity < cutoff:
                 channel.will_close = True
 
